@@ -67,6 +67,8 @@
 #include "ardour/audioengine.h"
 #include "ardour/vca_manager.h"
 
+#include "temporal/tempo.h"
+
 #include "mackie_control_protocol.h"
 
 #include "midi_byte_array.h"
@@ -327,8 +329,6 @@ MackieControlProtocol::get_sorted_stripables()
 				sorted.push_back (s);
 			}
 			break;
-		case Plugins:
-			break;
 		case Auxes: // in ardour, for now aux and buss are same. for mixbus, "Busses" are mixbuses, "Auxes" are ardour buses
 #ifdef MIXBUS
 			if (!s->mixbus() && !is_track(s) && !s->presentation_info().hidden())
@@ -339,7 +339,7 @@ MackieControlProtocol::get_sorted_stripables()
 				sorted.push_back (s);
 			}
 			break;
-		case Hidden: // Show all the tracks we have hidden
+		case Outputs: // Show all the tracks we have hidden
 			if (s->presentation_info().hidden()) {
 				// maybe separate groups
 				sorted.push_back (s);
@@ -350,6 +350,14 @@ MackieControlProtocol::get_sorted_stripables()
 				sorted.push_back (s);
 			}
 			break;
+		case AudioInstr:
+			if (has_instrument (s)){
+				sorted.push_back (s);
+			}
+			break;
+		case Inputs:
+			// nothing to do right now
+			break;
 		}
 	}
 
@@ -832,6 +840,8 @@ MackieControlProtocol::set_device (const string& device_name, bool force)
 		ARDOUR::AudioEngine::instance()->PortConnectedOrDisconnected.connect (port_connection, MISSING_INVALIDATOR, boost::bind (&MackieControlProtocol::connection_handler, this, _1, _2, _3, _4, _5), this);
 	}
 
+	build_button_map();
+
 	if (create_surfaces ()) {
 		return -1;
 	}
@@ -1012,7 +1022,7 @@ MackieControlProtocol::close()
  *  contains a state node for the device, it will deleted and replaced.
  */
 void
-MackieControlProtocol::update_configuration_state ()
+MackieControlProtocol::update_configuration_state () const
 {
 	/* CALLER MUST HOLD SURFACES LOCK */
 
@@ -1028,15 +1038,15 @@ MackieControlProtocol::update_configuration_state ()
 
 	XMLNode* snode = new XMLNode (X_("Surfaces"));
 
-	for (Surfaces::iterator s = surfaces.begin(); s != surfaces.end(); ++s) {
-		snode->add_child_nocopy ((*s)->get_state());
+	for (auto const & s : surfaces) {
+		snode->add_child_nocopy (s->get_state());
 	}
 
 	devnode->add_child_nocopy (*snode);
 }
 
 XMLNode&
-MackieControlProtocol::get_state()
+MackieControlProtocol::get_state() const
 {
 	XMLNode& node (ControlProtocol::get_state());
 
@@ -1153,9 +1163,9 @@ MackieControlProtocol::set_state (const XMLNode & node, int version)
 string
 MackieControlProtocol::format_bbt_timecode (samplepos_t now_sample)
 {
-	Timecode::BBT_Time bbt_time;
+	Temporal::BBT_Time bbt_time;
 
-	session->bbt_time (now_sample, bbt_time);
+	bbt_time = Temporal::TempoMap::fetch()->bbt_at (timepos_t (now_sample));
 
 	// The Mackie protocol spec is built around a BBT time display of
 	//
@@ -1501,6 +1511,12 @@ MackieControlProtocol::build_button_map ()
 
 #define DEFINE_BUTTON_HANDLER(b,p,r) button_map.insert (pair<Button::ID,ButtonHandlers> ((b), ButtonHandlers ((p),(r))));
 
+	if(!button_map.empty()) {
+		button_map.clear();
+	}
+
+	build_device_specific_button_map();
+
 	DEFINE_BUTTON_HANDLER (Button::Track, &MackieControlProtocol::track_press, &MackieControlProtocol::track_release);
 	DEFINE_BUTTON_HANDLER (Button::Send, &MackieControlProtocol::send_press, &MackieControlProtocol::send_release);
 	DEFINE_BUTTON_HANDLER (Button::Pan, &MackieControlProtocol::pan_press, &MackieControlProtocol::pan_release);
@@ -1568,6 +1584,38 @@ MackieControlProtocol::build_button_map ()
 	DEFINE_BUTTON_HANDLER (Button::MasterFaderTouch, &MackieControlProtocol::master_fader_touch_press, &MackieControlProtocol::master_fader_touch_release);
 }
 
+void
+MackieControlProtocol::build_device_specific_button_map()
+{
+	/* this maps our device-dependent button codes to the methods that handle them.
+	 */
+
+#define DEFINE_BUTTON_HANDLER(b,p,r) button_map.insert (pair<Button::ID,ButtonHandlers> ((b), ButtonHandlers ((p),(r))));
+
+	if (_device_info.is_platformMp()) {
+		DEFINE_BUTTON_HANDLER (Button::Marker, &MackieControlProtocol::flip_window_press, &MackieControlProtocol::flip_window_release);
+	}
+
+	if(_device_info.is_proG2()) {
+		DEFINE_BUTTON_HANDLER (Button::View, &MackieControlProtocol::user_press, &MackieControlProtocol::user_release);
+		DEFINE_BUTTON_HANDLER (Button::Trim, &MackieControlProtocol::send_press, &MackieControlProtocol::send_release);
+		DEFINE_BUTTON_HANDLER (Button::Touch, &MackieControlProtocol::open_press, &MackieControlProtocol::open_release);
+		DEFINE_BUTTON_HANDLER (Button::Latch, &MackieControlProtocol::flip_window_press, &MackieControlProtocol::flip_window_release);
+		DEFINE_BUTTON_HANDLER (Button::Save, &MackieControlProtocol::prog2_vst_press, &MackieControlProtocol::prog2_vst_release);
+		DEFINE_BUTTON_HANDLER (Button::Undo, &MackieControlProtocol::master_press, &MackieControlProtocol::master_release);
+		DEFINE_BUTTON_HANDLER (Button::Cancel, &MackieControlProtocol::prog2_clear_solo_press, &MackieControlProtocol::prog2_clear_solo_release);
+		DEFINE_BUTTON_HANDLER (Button::Enter, &MackieControlProtocol::shift_press, &MackieControlProtocol::shift_release);
+		DEFINE_BUTTON_HANDLER (Button::Marker, &MackieControlProtocol::prog2_left_press, &MackieControlProtocol::prog2_left_release);
+		DEFINE_BUTTON_HANDLER (Button::Nudge, &MackieControlProtocol::prog2_right_press, &MackieControlProtocol::prog2_right_release);
+		DEFINE_BUTTON_HANDLER (Button::Replace, &MackieControlProtocol::prev_marker_press, &MackieControlProtocol::prev_marker_release);
+		DEFINE_BUTTON_HANDLER (Button::Click, &MackieControlProtocol::prog2_marker_press, &MackieControlProtocol::prog2_marker_release);
+		DEFINE_BUTTON_HANDLER (Button::ClearSolo, &MackieControlProtocol::next_marker_press, &MackieControlProtocol::next_marker_release);
+		DEFINE_BUTTON_HANDLER (Button::Shift, &MackieControlProtocol::prog2_undo_press, &MackieControlProtocol::prog2_undo_release);
+		DEFINE_BUTTON_HANDLER (Button::Option, &MackieControlProtocol::redo_press, &MackieControlProtocol::redo_release);
+		DEFINE_BUTTON_HANDLER (Button::Ctrl, &MackieControlProtocol::prog2_save_press, &MackieControlProtocol::prog2_save_release);
+	}
+}
+
 void
 MackieControlProtocol::handle_button_event (Surface& surface, Button& button, ButtonState bs)
 {
@@ -1681,7 +1729,7 @@ MackieControlProtocol::midi_input_handler (IOCondition ioc, MIDI::Port* port)
 
 	if (ioc & IO_IN) {
 
-		// DEBUG_TRACE (DEBUG::MackieControl, string_compose ("something happend on  %1\n", port->name()));
+		// DEBUG_TRACE (DEBUG::MackieControl, string_compose ("something happened on  %1\n", port->name()));
 
 		/* Devices using regular JACK MIDI ports will need to have
 		   the x-thread FIFO drained to avoid burning endless CPU.
@@ -1816,11 +1864,15 @@ MackieControlProtocol::set_view_mode (ViewMode m)
 void
 MackieControlProtocol::display_view_mode ()
 {
-	Glib::Threads::Mutex::Lock lm (surfaces_lock);
+	{
+		Glib::Threads::Mutex::Lock lm (surfaces_lock);
 
-	for (Surfaces::iterator s = surfaces.begin(); s != surfaces.end(); ++s) {
-		(*s)->update_view_mode_display (true);
+		for (Surfaces::iterator s = surfaces.begin(); s != surfaces.end(); ++s) {
+			(*s)->update_view_mode_display (true);
+		}
 	}
+
+	update_global_button (Button::View, (view_mode() == MackieControlProtocol::Mixer) ? on : off);
 }
 
 void
@@ -2310,6 +2362,13 @@ MackieControlProtocol::is_midi_track (boost::shared_ptr<Stripable> r) const
 	return boost::dynamic_pointer_cast<MidiTrack>(r) != 0;
 }
 
+bool
+MackieControlProtocol::has_instrument (boost::shared_ptr<Stripable> r) const
+{
+	boost::shared_ptr<MidiTrack> mt = boost::dynamic_pointer_cast<MidiTrack>(r);
+	return mt && mt->the_instrument();
+}
+
 bool
 MackieControlProtocol::is_mapped (boost::shared_ptr<Stripable> r) const
 {
